#include "https.h"

int at_comms_init()
{
	int err;

	err = at_cmd_init();
	if (err)
	{
		printk("Failed to initialize AT commands, err %d\n", err);
		return err;
	}

	err = at_notif_init();
	if (err)
	{
		printk("Failed to initialize AT notifications, err %d\n", err);
		return err;
    }

	return 0;
}


int tls_setup(int fd)
{
	int err;
	int verify;

	/* Security tag that we have provisioned the certificate with */
	const sec_tag_t tls_sec_tag[] = {
		TLS_SEC_TAG,
	};

	/* Set up TLS peer verification */
	enum
	{
		NONE = 0,
		OPTIONAL = 1,
		REQUIRED = 2,
	};

	verify = REQUIRED;

	err = setsockopt(fd, SOL_TLS, TLS_PEER_VERIFY, &verify, sizeof(verify));
	if (err)
	{
		printk("Failed to setup peer verification, err %d\n", errno);
		return err;
	}

	/* Associate the socket with the security tag
	 * we have provisioned the certificate with.
	 */
	err = setsockopt(fd, SOL_TLS, TLS_SEC_TAG_LIST, tls_sec_tag,
					 sizeof(tls_sec_tag));
	if (err)
	{
		printk("Failed to setup TLS sec tag, err %d\n", errno);
		return err;
	}

	return 0;
}

char* parseRecv(const char *input)
{
	char *pch = NULL;

    //search for the end of the response headers for the meat of the content
	pch = strstr(input, "\r\n\r\n");

    //return pointer to start of what we want
	return pch;
}
void get_request(const char *path, const char *host)
{
	int err;
	int bytes, sockfd, totalBytes = 0;
	size_t off;
	struct addrinfo *res;
	struct addrinfo hints;

	char sendBuff[SEND_BUFF_SIZE];
	char recBuff[RECV_BUFF_SIZE];
	int sendLength = sprintf(sendBuff,
							 GET_TEMPLATE,
							 path,
							 host);

	memset(&hints, 0, sizeof(hints));
	hints.ai_family = AF_INET;
	hints.ai_socktype = SOCK_STREAM;

	printk("HTTP GET started..\n\r");

	err = getaddrinfo(host, NULL, &hints, &res);
	if (err)
	{
		printk("getaddrinfo errno %d\n", errno);
		/* No clean up needed, just return */
		return;
	}

	printk("Address info received\n\r");
	((struct sockaddr_in *)res->ai_addr)->sin_port = htons(HTTPS_PORT);

	sockfd = socket(AF_INET, SOCK_STREAM, IPPROTO_TLS_1_2);
	if (sockfd == -1)
	{
		printk("Failed to open socket\n");
		return;
	}
	enum
	{
		NONE = 0,
		OPTIONAL = 1,
		REQUIRED = 2,
	};

	int verify = OPTIONAL;

	err = setsockopt(sockfd, SOL_TLS, TLS_PEER_VERIFY, &verify,
					 sizeof(verify));
	if (err)
	{
		printk("setsockopt err: %d\n", errno);
	}

	err = connect(sockfd, (struct sockaddr *)res->ai_addr, sizeof(struct sockaddr_in));
	if (err > 0)
	{
		printk("connect err: %d\n", errno);
		return; //goto clean_up;
	}

	off = 0;
	printk("%s\n", sendBuff);
	do
	{
		bytes = send(sockfd, sendBuff, sendLength - off, 0);
		if (bytes < 0)
		{
			printk("send() failed, err %d\n", errno);
			return; //goto clean_up;
		}
		off += bytes;
	} while (off < sendLength);

	printk("Sent %d bytes\n", off);

	do
	{
		/* TODO: make a proper timeout *
            * Current solution will just hang 
            * until remote side closes connection */
		bytes = recv(sockfd, recBuff, RECV_BUFF_SIZE, 0);
		totalBytes += bytes;
		printk("total number of bytes: %d\n", totalBytes);
		printk("num bytes: %d\n", bytes);
		if (bytes < 0)
		{
			printk("\nrecv errno: %d\n", errno);
			break;
		}
		printk("%s\n", recBuff);
	} while (bytes > 0);

	printk("Received %d bytes\n", off);

	printk("\n\rFinished. Closing socket\r\n");
	freeaddrinfo(res);
	err = close(sockfd);
}
void post_request(const char *path, const char *host, char *sendData)
{
	int err;
	char *p = NULL; //pointer to response
	int bytes, sockfd, totalBytes = 0;
	size_t off;
	struct addrinfo *res;
	struct addrinfo hints;

	char sendBuff[SEND_BUFF_SIZE];
	char recBuff[RECV_BUFF_SIZE];
	memset(&recBuff, 0, sizeof(recBuff));
	int sendLength = snprintf(sendBuff,
							  500,
							  POST_TEMPLATE,
							  path,
							  host,
							  strlen(sendData),
							  SEND_DATA);

	memset(&hints, 0, sizeof(hints));
	hints.ai_family = AF_INET;
	hints.ai_socktype = SOCK_STREAM;

	printk("HTTP POST started..\n\r");

	err = getaddrinfo(host, NULL, &hints, &res);
	if (err)
	{
		printk("getaddrinfo errno %d\n", errno);
		/* No clean up needed, just return */
		return;
	}

	printk("Address info received\n\r");
	((struct sockaddr_in *)res->ai_addr)->sin_port = htons(HTTPS_PORT);

	sockfd = socket(AF_INET, SOCK_STREAM, IPPROTO_TLS_1_2);

	err = tls_setup(sockfd);
	if (err)
	{
		return;
	}
	enum
	{
		NONE = 0,
		OPTIONAL = 1,
		REQUIRED = 2,
	};

	int verify = OPTIONAL;

	err = setsockopt(sockfd, SOL_TLS, TLS_PEER_VERIFY, &verify,
					 sizeof(verify));
	if (err)
	{
		printk("setsockopt err: %d\n", errno);
	}

	err = connect(sockfd, (struct sockaddr *)res->ai_addr, sizeof(struct sockaddr_in));
	if (err > 0)
	{
		printk("connect err: %d\n", errno);
		return; //goto cleanup
	}

	off = 0;
	do
	{
		bytes = send(sockfd, sendBuff, sendLength - off, 0);
		if (bytes < 0)
		{
			printk("send() failed, err %d\n", errno);
			return;
		}
		off += bytes;
	} while (off < sendLength);

	printk("Sent %d bytes\n", off);

	do
	{
		/* TODO: make a proper timeout *
		 * Current solution will just hang 
		 * until remote side closes connection */
		bytes = recv(sockfd, recBuff, RECV_BUFF_SIZE, 0);
		totalBytes += bytes;
		if (bytes < 0)
		{
			printk("\nrecv errno: %d\n", errno);
			break;
		}
		printk("%s\n", recBuff);
	} while (bytes > 0);

	printk("%s\n", recBuff);
	printk("Received %d bytes\n", off);

	//	/* Print HTTP response */
	//	p = strstr(recBuff, "\r\n");
	//	if (p) {
	//		off = p - recBuff;
	//		recBuff[off + 1] = '\0';
	//		printk("\n>\t %s\n\n", recBuff);
	//	}

	printk("\n\rFinished. Closing socket\r\n");
	freeaddrinfo(res);
	err = close(sockfd);

	p = parseRecv(recBuff);
	printk("Filtered response: %s", p);
}